;nyquist plug-in
;version 3
;type process
;categories "http://lv2plug.in/ns/lv2core/#UtilityPlugin"
;name "DC offst..."
;action "Asjusting DC offset..."
;info "by Steve Daulton (www.easyspacepro.com).\nReleased under GPL v2.\n"

;; dc-offset.ny by Steve Daulton October 2012
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html 
;;
;; For information about writing and modifying Nyquist plug-ins:
;; http://wiki.audacityteam.org/wiki/Nyquist_Plug-ins_Reference

;control mode "Apply or remove DC offset" choice "Remove offset,Add offset,Remove then Add" 0
;control r-type "Offset removal method" choice "Dynamic,Absolute,HP Filter,Abs + Filter,Equal +/- Peaks" 0
;control offset "How much offset to add (+/- 1)" real "" 0 -1 1

(defun remove-offset (method sig)
  (case method
    (0 (if (> (get-duration 1) 0.1)
           (multichan-expand #'hybrid-dc-fix s)
           (highpass2 (multichan-expand #'zero-abs sig) 20)))
    (1 (multichan-expand #'zero-abs sig))
    (2 (highpass2 sig 20))
    (3 (highpass2 (multichan-expand #'zero-abs sig) 20))
    (t (multichan-expand #'eq-peaks sig))))

;;; Hybrid method
(defun hybrid-dc-fix (sig)
  (let* ((srate (snd-srate sig))
         (step (round (/ srate 10))))
    (highpass2
      (sum sig
        (mult -1
          (snd-avg sig step step op-average)))
      20)))

;;; Absolute DC offset
(defun zero-abs (sig)
  (diff sig (calc-dc sig)))

;;; calculate dc offset
(defun calc-dc (sig)
    (sref (sound (mult (integrate sig)
                       (/ (get-duration 1))))
          (/ (1- len) *sound-srate*)))

;;; Equal peaks
(defun eq-peaks (sig)
  (let ((pos (- (peak (sum 100 sig) ny:all) 100))
         (neg (- 100 (peak (sum -100 sig) ny:all))))
    (sum sig (/ (- (+ pos neg)) 2))))
    

;;; workaround bug 425
(defun s-sum (sig val)
  (multichan-expand #'sum sig val))

(case mode
  (0 (remove-offset r-type s))
  (1 (s-sum s offset))
  (t (s-sum (remove-offset r-type s) offset)))
  